@@ -1,19 +1,21 @@
 require 'jbuilder/jbuilder'
+require 'jbuilder/blank'
 require 'jbuilder/key_formatter'
 require 'jbuilder/errors'
 require 'multi_json'
+require 'ostruct'
 
 class Jbuilder
-  @@key_formatter = KeyFormatter.new
+  @@key_formatter = nil
   @@ignore_nil    = false
 
-  def initialize(options = {}, &block)
+  def initialize(options = {})
     @attributes = {}
 
-    @key_formatter = options.fetch(:key_formatter){ @@key_formatter.clone }
+    @key_formatter = options.fetch(:key_formatter){ @@key_formatter ? @@key_formatter.clone : nil}
     @ignore_nil = options.fetch(:ignore_nil, @@ignore_nil)
 
-    yield self if block
+    yield self if ::Kernel.block_given?
   end
 
   # Yields a builder and automatically turns the result into a JSON string
@@ -21,14 +23,15 @@ class Jbuilder
     new(*args, &block).target!
   end
 
-  BLANK = ::Object.new
+  BLANK = Blank.new
+  NON_ENUMERABLES = [ ::Struct, ::OpenStruct ].to_set
 
-  def set!(key, value = BLANK, *args, &block)
-    result = if block
-      if BLANK != value
+  def set!(key, value = BLANK, *args)
+    result = if ::Kernel.block_given?
+      if !_blank?(value)
         # json.comments @post.comments { |comment| ... }
         # { "comments": [ { ... }, { ... } ] }
-        _scope{ array! value, &block }
+        _scope{ array! value, &::Proc.new }
       else
         # json.comments { ... }
         # { "comments": ... }
@@ -45,7 +48,7 @@ class Jbuilder
         # { "age": 32 }
         value
       end
-    elsif _mapable_arguments?(value, *args)
+    elsif _is_collection?(value)
       # json.comments @post.comments, :content, :created_at
       # { "comments": [ { "content": "hello", "created_at": "..." }, { "content": "world", "created_at": "..." } ] }
       _scope{ array! value, *args }
@@ -58,8 +61,13 @@ class Jbuilder
     _set_value key, result
   end
 
-  alias_method :method_missing, :set!
-  private :method_missing
+  def method_missing(*args)
+    if ::Kernel.block_given?
+      set! *args, &::Proc.new
+    else
+      set! *args
+    end
+  end
 
   # Specifies formatting to be applied to the key. Passing in a name of a function
   # will cause that function to be called on the key.  So :upcase will upper case
@@ -173,11 +181,11 @@ class Jbuilder
   #   json.array! [1, 2, 3]
   #
   #   [1,2,3]
-  def array!(collection = [], *attributes, &block)
+  def array!(collection = [], *attributes)
     array = if collection.nil?
       []
-    elsif block
-      _map_collection(collection, &block)
+    elsif ::Kernel.block_given?
+      _map_collection(collection, &::Proc.new)
     elsif attributes.any?
       _map_collection(collection) { |element| extract! element, *attributes }
     else
@@ -206,15 +214,15 @@ class Jbuilder
   #   json.(@person, :name, :age)
   def extract!(object, *attributes)
     if ::Hash === object
-      _extract_hash_values(object, *attributes)
+      _extract_hash_values(object, attributes)
     else
-      _extract_method_values(object, *attributes)
+      _extract_method_values(object, attributes)
     end
   end
 
-  def call(object, *attributes, &block)
-    if block
-      array! object, &block
+  def call(object, *attributes)
+    if ::Kernel.block_given?
+      array! object, &::Proc.new
     else
       extract! object, *attributes
     end
@@ -244,72 +252,73 @@ class Jbuilder
 
   private
 
-  def _extract_hash_values(object, *attributes)
+  def _extract_hash_values(object, attributes)
     attributes.each{ |key| _set_value key, object.fetch(key) }
   end
 
-  def _extract_method_values(object, *attributes)
+  def _extract_method_values(object, attributes)
     attributes.each{ |key| _set_value key, object.public_send(key) }
   end
 
-  def _merge_block(key, &block)
-    current_value = _read(key, {})
+  def _merge_block(key)
+    current_value = _blank? ? BLANK : @attributes.fetch(_key(key), BLANK)
     raise NullError.build(key) if current_value.nil?
-    value = _scope{ yield self }
-    value.nil? ? value : _merge_values(current_value, value)
+    new_value = _scope{ yield self }
+    _merge_values(current_value, new_value)
   end
 
-  def _read(key, default = nil)
-    @attributes.fetch(_key(key)){ default }
-  end
-
-  def _write(key, value)
-    @attributes[_key(key)] = value
+  def _merge_values(current_value, updates)
+    if _blank?(updates)
+      current_value
+    elsif _blank?(current_value) || updates.nil?
+      updates
+    elsif ::Array === updates
+      ::Array === current_value ? current_value + updates : updates
+    elsif ::Hash === current_value
+      current_value.merge(updates)
+    else
+      raise "Can't merge #{updates.inspect} with #{current_value.inspect}"
+    end
   end
 
   def _key(key)
-    @key_formatter.format(key)
+    @key_formatter ? @key_formatter.format(key) : key.to_s
   end
 
   def _set_value(key, value)
     raise NullError.build(key) if @attributes.nil?
-    return if @ignore_nil && value.nil?
-    _write key, value
+    raise ArrayError.build(key) if ::Array === @attributes
+    return if @ignore_nil && value.nil? or _blank?(value)
+    @attributes = {} if _blank?
+    @attributes[_key(key)] = value
   end
 
   def _map_collection(collection)
     collection.map do |element|
       _scope{ yield element }
-    end
+    end - [BLANK]
   end
 
   def _scope
     parent_attributes, parent_formatter = @attributes, @key_formatter
-    @attributes = {}
+    @attributes = BLANK
     yield
     @attributes
   ensure
     @attributes, @key_formatter = parent_attributes, parent_formatter
   end
 
-  def _mapable_arguments?(value, *args)
-    value.respond_to?(:map)
+  def _is_collection?(object)
+    _object_respond_to?(object, :map, :count) && NON_ENUMERABLES.none?{ |klass| klass === object }
   end
 
-  def _merge_values(attributes, hash_or_array)
-    attributes = attributes.dup
+  def _blank?(value=@attributes)
+    BLANK == value
+  end
 
-    if ::Array === hash_or_array
-      attributes = [] unless ::Array === attributes
-      attributes.concat hash_or_array
-    else
-      attributes.update hash_or_array
-    end
-
-    attributes
+  def _object_respond_to?(object, *methods)
+    methods.all?{ |m| object.respond_to?(m) }
   end
 end
 
-require 'jbuilder/jbuilder_template' if defined?(ActionView::Template)
-require 'jbuilder/dependency_tracker'
-require 'jbuilder/railtie' if defined?(Rails::VERSION::MAJOR) && Rails::VERSION::MAJOR == 4
+require 'jbuilder/railtie' if defined?(Rails)
