@@ -9,34 +9,16 @@ class JbuilderTemplate < Jbuilder
 
   self.template_lookup_options = { handlers: [:jbuilder] }
 
-  def initialize(context, *args, &block)
+  def initialize(context, *args)
     @context = context
-    super(*args, &block)
+    super *args
   end
 
-  def partial!(name_or_options, locals = {})
-    case name_or_options
-    when ::Hash
-      # partial! partial: 'name', locals: { foo: 'bar' }
-      options = name_or_options
+  def partial!(*args)
+    if args.one? && _is_active_model?(args.first)
+      _render_active_model_partial args.first
     else
-      # partial! 'name', foo: 'bar'
-      options = { partial: name_or_options, locals: locals }
-      as = locals.delete(:as)
-      options[:as] = as if as.present?
-      options[:collection] = locals[:collection] if locals.key?(:collection)
-    end
-
-    _render_partial_with_options options
-  end
-
-  def array!(collection = [], *attributes, &block)
-    options = attributes.extract_options!
-
-    if options.key?(:partial)
-      partial! options[:partial], options.merge(collection: collection)
-    else
-      super
+      _render_explicit_partial *args
     end
   end
 
@@ -48,7 +30,7 @@ class JbuilderTemplate < Jbuilder
   #   json.cache! ['v1', @person], expires_in: 10.minutes do
   #     json.extract! @person, :name, :age
   #   end
-  def cache!(key=nil, options={}, &block)
+  def cache!(key=nil, options={})
     if @context.controller.perform_caching
       value = ::Rails.cache.fetch(_cache_key(key, options), options) do
         _scope { yield self }
@@ -60,7 +42,7 @@ class JbuilderTemplate < Jbuilder
     end
   end
 
-  # Conditionally catches the json depending in the condition given as first parameter. Has the same
+  # Conditionally caches the json depending in the condition given as first parameter. Has the same
   # signature as the `cache` helper method in `ActionView::Helpers::CacheHelper` and so can be used in
   # the same way.
   #
@@ -69,11 +51,31 @@ class JbuilderTemplate < Jbuilder
   #   json.cache_if! !admin?, @person, expires_in: 10.minutes do
   #     json.extract! @person, :name, :age
   #   end
-  def cache_if!(condition, *args, &block)
-    condition ? cache!(*args, &block) : yield
+  def cache_if!(condition, *args)
+    condition ? cache!(*args, &::Proc.new) : yield
   end
 
-  protected
+  def array!(collection = [], *args)
+    options = args.first
+
+    if args.one? && _partial_options?(options)
+      partial! options.merge(collection: collection)
+    else
+      super
+    end
+  end
+
+  def set!(name, object = BLANK, *args)
+    options = args.first
+
+    if args.one? && _partial_options?(options)
+      _set_inline_partial name, object, options
+    else
+      super
+    end
+  end
+
+  private
 
   def _render_partial_with_options(options)
     options.reverse_merge! locals: {}
@@ -81,6 +83,7 @@ class JbuilderTemplate < Jbuilder
     as = options[:as]
 
     if as && options.key?(:collection)
+      as = as.to_sym
       collection = options.delete(:collection)
       locals = options.delete(:locals)
       array! collection do |member|
@@ -105,8 +108,6 @@ class JbuilderTemplate < Jbuilder
     ::ActiveSupport::Cache.expand_cache_key(key, :jbuilder)
   end
 
-  private
-
   def _fragment_name_with_digest(key, options)
     if @context.respond_to?(:cache_fragment_name)
       # Current compatibility, fragment_name_with_digest is private again and cache_fragment_name
@@ -120,10 +121,50 @@ class JbuilderTemplate < Jbuilder
     end
   end
 
-  def _mapable_arguments?(value, *args)
-    return true if super
-    options = args.last
-    ::Hash === options && options.key?(:as)
+  def _partial_options?(options)
+    ::Hash === options && options.key?(:as) && options.key?(:partial)
+  end
+
+  def _is_active_model?(object)
+    object.class.respond_to?(:model_name) && object.respond_to?(:to_partial_path)
+  end
+
+  def _set_inline_partial(name, object, options)
+    value = if object.nil?
+      []
+    elsif _is_collection?(object)
+      _scope{ _render_partial_with_options options.merge(collection: object) }
+    else
+      locals = ::Hash[options[:as], object]
+      _scope{ _render_partial options.merge(locals: locals) }
+    end
+
+    set! name, value
+  end
+
+  def _render_explicit_partial(name_or_options, locals = {})
+    case name_or_options
+    when ::Hash
+      # partial! partial: 'name', foo: 'bar'
+      options = name_or_options
+    else
+      # partial! 'name', locals: {foo: 'bar'}
+      if locals.one? && (locals.keys.first == :locals)
+        options = locals.merge(partial: name_or_options)
+      else
+        options = { partial: name_or_options, locals: locals }
+      end
+      # partial! 'name', foo: 'bar'
+      as = locals.delete(:as)
+      options[:as] = as if as.present?
+      options[:collection] = locals[:collection] if locals.key?(:collection)
+    end
+
+    _render_partial_with_options options
+  end
+
+  def _render_active_model_partial(object)
+    @context.render object, json: self
   end
 end
 
@@ -137,5 +178,3 @@ class JbuilderHandler
       json.target! unless (__already_defined && __already_defined != "method")}
   end
 end
-
-ActionView::Template.register_template_handler :jbuilder, JbuilderHandler
